(******************************************************************************************************)
(*												      *)
(*	Dr. William T. Verts (C) April 28, 1996							      *)
(*												      *)
(*	Simple driver program to test VGA 320x200 256-color graphics mode unit.			      *)
(*												      *)
(******************************************************************************************************)
{$R-}
{$I+}
{$N-}
{$E-}

Program Test_VGA_320 ;

Uses	Crt,
	Dos,
	VGA_320 ;

Const	Bell		= ^G ;

Var	Save_Palette	: Color_Table ;
	I		: LongInt ;
	X1, Y1		: LongInt ;
	X2, Y2		: LongInt ;
	Radius		: LongInt ;
	C		: Byte ;
	Width		: Integer ;
	Height		: Integer ;
	R, G, B		: Byte ;
	P_Color		: Color_Table ;
	P_Gray		: Color_Table ;
	P_Spin		: Color_Table ;
	Angle		: Real	;
	Steps		: Integer ;
	Done_Flag	: Boolean ;
	Wait		: Integer ;
	CH		: Char ;
	Background	: Boolean ;

(******************************************************************************************************)
(* Wait for the user to strike a key.								      *)

Procedure Continue ;
Begin	(***)
	While KeyPressed Do ReadKey ;
	Write	(Bell) ;
	While Not KeyPressed Do ;
	While KeyPressed Do ReadKey ;
End ;

(******************************************************************************************************)
(* Step through the color and gray palettes (1/2 second delay between each) until key is hit.	      *)

Procedure Cycle_Colors ;
Begin	(***)
	Write	(Bell) ;

	While Not KeyPressed Do
		Begin	(***)
			If Not KeyPressed Then
				Begin	(***)
					Put_Palette (P_Gray) ;
					Delay (500) ;
				End ;

			If Not KeyPressed Then
				Begin	(***)
					Put_Palette (P_Color) ;
					Delay (500) ;
				End ;
		End ;

	While KeyPressed Do ReadKey ;
End ;

(******************************************************************************************************)

Begin	(*!*)
	TextMode	(C80) ;
	TextColor	(White) ;
	TextBackground	(Blue) ;
	ClrScr		;
			(*12345678901234567890123456789012345678901234567890123456789012345678901234567890*)
	Writeln		;
	Writeln		('             Demonstration Program for VGA 320x200 256-color mode') ;
	Writeln		('             Copyright: Dr. William T. Verts (C) April 28, 1996') ;
	Writeln		;
	Writeln		('             Under most circumstances, the current demonstration') ;
	Writeln		('             mode can be halted by hitting any key.  In the') ;
	Writeln		('             demo for the orbiting planet and the rectangles,') ;
	Writeln		('             the speed may be changed by hitting one of the') ;
	Writeln		('             digits [0..9].  Also, "B" changes the background') ;
	Writeln		('             in the planets demo, and changes the direction of') ;
	Writeln		('             the rectangles to backwards from current motion.') ;
	Continue ;

	(*--------------------------------------------------------------------*)
	(* Switch into graphics mode (320x200 256 color)		      *)
	(* and save the palette (it will be restored before the program ends) *)

	InitGraph ;
	Get_Palette (Save_Palette) ;

	(*--------------------------------------------------------------------*)
	(* Build up a color palette (216 entries).			      *)

	FillChar (P_Color, SizeOf(P_Color), Zero) ;

	For R := 0 To 5 Do
		For G := 0 To 5 Do
			For B := 0 To 5 Do
				With P_Color[((R * 6) + G) * 6 + B] Do
					Begin	(***)
						Red	:= R * 51 ;
						Green	:= G * 51 ;
						Blue	:= B * 51 ;
					End ;

	(*--------------------------------------------------------------------*)
	(* Build up a gray scale palette (256 entries).			      *)

	FillChar (P_Gray,  SizeOf(P_Gray),  Zero) ;

	For I := 0 To 255 Do
		With P_Gray[I] Do
			Begin	(***)
				Red	:= I ;
				Green	:= I ;
				Blue	:= I ;
			End ;

	(*--------------------------------------------------------------------*)
	(* Make the color palette the active palette.			      *)

	Put_Palette (P_Color) ;

	(*--------------------------------------------------------------------*)
	(* Generate random rectangles of random colors, with a		      *)
	(* border and the diagonals of a different random color.	      *)
	(* When complete, cycle between color and gray palettes.	      *)

	While Not KeyPressed Do
		Begin	(***)
			X1	:= Random(GetMaxX + 1) ;
			Y1	:= Random(GetMaxY + 1) ;
			X2	:= Random(GetMaxX + 1) ;
			Y2	:= Random(GetMaxY + 1) ;
			C	:= Random(256) ;
			Filled_Rectangle (X1, Y1, X2, Y2, C) ;
			C	:= Random(256) ;
			Line	(X1, Y1, X2, Y2, C) ;
			Line	(X1, Y2, X2, Y1, C) ;
			Rectangle (X1, Y1, X2, Y2, C) ;
		End ;

	Continue ;
	Cycle_Colors ;

	(*--------------------------------------------------------------------*)
	(* Generate random circles of random colors and random		      *)
	(* radii, with a border of a different random color.		      *)
	(* When complete, cycle between color and gray palettes.	      *)

	ClearDevice ;
	Put_Palette (P_Color) ;

	While Not KeyPressed Do
		Begin	(***)
			X1	:= Random(GetMaxX + 1) ;
			Y1	:= Random(GetMaxY + 1) ;
			Radius	:= Random(30) + 4 ;
			C	:= Random(256) ;
			Filled_Circle (X1, Y1, Radius, C) ;
			C	:= Random(256) ;
			Circle	(X1, Y1, Radius, C) ;
		End ;

	Continue ;
	Cycle_Colors ;

	(*--------------------------------------------------------------------*)
	(* Generate a grid of 16x16 rectangles, each of a different color.    *)
	(* This will show all 256 colors on screen at the same time.	      *)
	(* When complete, cycle between color and gray palettes.	      *)

	ClearDevice ;
	Put_Palette (P_Color) ;
	Width	:= Succ(GetMaxX) Div 16 ;
	Height	:= Succ(GetMaxY) Div 16 ;

	For C := 0 To 255 Do
		Begin	(***)
			X1	:= (C Mod 16) * Width ;
			X2	:= X1 + Width - 1 ;
			Y1	:= (C Div 16) * Height ;
			Y2	:= Y1 + Height - 1 ;
			Filled_Rectangle (X1, Y1, X2, Y2, C) ;
		End ;

	Continue ;
	Cycle_Colors ;

	(*--------------------------------------------------------------------*)
	(* Generate a simple animation using the palette to step between the  *)
	(* frames.  Animation is a yellow sun in a black sky, with a blue-    *)
	(* green planet orbiting around it.  First step is to blank out the   *)
	(* screen by clearing all entries in the palette, then to plot the    *)
	(* individual positions of the planet in successive pairs of colors.  *)
	(* When complete, the screen will contain all of the "images" where   *)
	(* each image is activated by storing a non-black color into the      *)
	(* correct places in the palette.  The interactive loop animates the  *)
	(* images by updating the palette in a loop.  User has speed control. *)
	(* Notice that the planet is not always "whole" since it is so        *)
	(* big that planets from successive images overlap -- a problem with  *)
	(* this technique.						      *)

	FillChar (P_Spin, SizeOf(P_Spin), Zero) ;
	Put_Palette (P_Spin) ;
	Fill_Screen (254) ;

	Steps	:= 50 ;
	Radius	:= 6 ;

	For I := 0 To Pred(Steps) Do
		Begin	(***)
			Angle	:= (I / Steps) * 2.0 * Pi ;
			X1	:= Round(GetMidX + (GetMidX - Radius * 3) * Cos(Angle)) ;
			Y1	:= Round(GetMidY - (GetMidY - Radius * 3) * Sin(Angle)) ;
			Filled_Circle (X1, Y1, Radius, I*2+1) ;
			Circle	      (X1, Y1, Radius, I*2+2) ;
		End ;

	For I := 0 To Pred(Steps) Do Vertical_Line (I+1, 1, 3, I*2+1) ;

	Rectangle (0, 0, Steps+1, 4, 255) ;
	Filled_Circle (GetMidX, GetMidY, 20, 255) ; (* Plot the sun at center *)

	(*--------------------------------------------------------------------*)

	Done_Flag	:= False ;
	I		:= 0 ;
	Wait		:= 100 ;
	Background	:= False ;

	Repeat	(***)
		(* Build up the palette for the next image *)
		FillChar (P_Spin, SizeOf(P_Spin), Zero) ;
		With P_Spin[2*I+1] Do Begin Red :=   0 ; Green := 255 ; Blue :=   0 End ; (* Planet Body *)
		With P_Spin[2*I+2] Do Begin Red :=   0 ; Green := 255 ; Blue := 255 End ; (* Planet Rim  *)
		With P_Spin[255  ] Do Begin Red := 255 ; Green := 255 ; Blue :=   0 End ; (* Sun         *)

		If Background Then
			With P_Spin[254  ] Do Begin Red :=   0 ; Green :=   0 ; Blue := 255 End ; (* Sky         *)

		(* Update the image *)
		Put_Palette (P_Spin) ;
		Delay	(Wait) ;

		(* Step to next image index *)
		I := (I + 1) Mod Steps ;

		(* Handle Interactive part *)
		While KeyPressed Do
			Begin	(***)
				CH	:= UpCase(ReadKey) ;

				Case CH Of
			'0'..'9' :	Wait := (Ord(CH) - Ord('0')) * 25 ;	(* Compute Wait Time   *)
			'B'	 :	Background := Not Background ;		(* Toggle Sky	       *)
			#27, 'Q',						(* Escape, Return,     *)
			#13, ' ' :	Done_Flag := True ;			(* Blank or Q quits    *)
			#0	 :	ReadKey ;				(* Flush function keys *)
			Else		Begin End ;				(* Ignore other keys   *)
				End ;	(* CASE *)
			End ;
	Until Done_Flag ;

	(*--------------------------------------------------------------------*)
	(* Same kind of problem as before, except that figures are concentric *)
	(* rectangles, giving a "marquee" effect.  Direction is selectable.   *)

	FillChar (P_Spin, SizeOf(P_Spin), Zero) ;
	Put_Palette (P_Spin) ;
	Fill_Screen (254) ;

	Steps		:= 20 ;
	For I := 0 To GetMaxY Div 2 Do Rectangle (I, I, GetMaxX-I, GetMaxY-I, (I Mod Steps)+1) ;

	Done_Flag	:= False ;
	I		:= 0 ;
	Wait		:= 100 ;
	Background	:= False ;

	Repeat	(***)
		(* Build up the palette for the next image *)
		FillChar (P_Spin, SizeOf(P_Spin), Zero) ;
		With P_Spin[I+1] Do Begin Red :=   0 ; Green := 255 ; Blue :=   0 End ;

		(* Update the image *)
		Put_Palette (P_Spin) ;
		Delay	(Wait) ;

		(* Step to next image index *)
		If Background Then I := (I - 1 + Steps) Mod Steps
			      Else I := (I + 1 + Steps) Mod Steps ;

		(* Handle Interactive part *)
		While KeyPressed Do
			Begin	(***)
				CH	:= UpCase(ReadKey) ;

				Case CH Of
			'0'..'9' :	Wait := (Ord(CH) - Ord('0')) * 25 ;	(* Compute Wait Time   *)
			'B'	 :	Background := Not Background ;		(* Toggle Direction    *)
			#27, 'Q',						(* Escape, Return,     *)
			#13, ' ' :	Done_Flag := True ;			(* Blank or Q quits    *)
			#0	 :	ReadKey ;				(* Flush function keys *)
			Else		Begin End ;				(* Ignore other keys   *)
				End ;	(* CASE *)
			End ;
	Until Done_Flag ;

	(*--------------------------------------------------------------------*)
	(* Turn off graphics mode, and return to text mode.		      *)

	Put_Palette (Save_Palette) ;
	CloseGraph ;
End.                          
